/* 
 * File:   BodyCollection.h
 * Author: daniel
 *
 * Created on August 22, 2010, 3:55 PM
 */

#ifndef _BODYCOLLECTION_H
#define	_BODYCOLLECTION_H

#include "Ray.h"
#include "Body.h"
//----------------------BodyCollection---------------------
// This class stores the collection of pointers to bodies and its main
// job is to get a ray as input, check if it's intersecting
// any of the bodies and return the closest one if there's
// a hit.
// The idea is that the BodyCollection stores the bodies in
// a way that makes the search efficient.
//---------------------------------------------------------
class BodyCollection{
public:
    BodyCollection(){}
    virtual ~BodyCollection(){} //each child needs to implement a storage specific dtor
    virtual void addBody(Body* body) = 0;

    //this checks if the ray intersects any of the bodies and sets the intersection
    //point and the normal at the closest intersection point.  It returns 0 if there was no intersection,
    //1 if the ray intersects from outside and -1 if it intersects from inside
    virtual int intersectRay(Ray& ray, Vector3d& interPoint, Vector3d& normal, Body*& body) = 0;

    //this checks if any of the bodies intersect the ray within dist distance from the ray origin
    virtual bool hasIntersection(Ray& ray, double distSq) const = 0;
};

#endif	/* _BODYCOLLECTION_H */

