#include "BasicCollection.h"

void BasicCollection::addBody(Body* body)
{
    bodies.push_back(body);
}

int BasicCollection::intersectRay(Ray& ray, Vector3d& interPoint, Vector3d& normal, Body*& body)
{
   int output = 0;
   double closestHit = 0;
   
   for (int i=0; i<bodies.size(); ++i)
   {
       Vector3d bodyNormal = Vector3d::Zero();
       Vector3d bodyPoint = Vector3d::Zero();
       int bodyOutput = bodies[i]->getGeometry()->intersectRay(ray, bodyPoint, bodyNormal);

       if (bodyOutput == 0) continue; //if we dont have a hit then go to the next body

       double hitDistSq = (bodyPoint - ray.getOrigin()).squaredNorm();

       //if this is the first hit or a closer hit
       if ((hitDistSq < closestHit) || (output == 0))
       {
           output = bodyOutput;
           closestHit = hitDistSq;
           interPoint = bodyPoint;
           normal = bodyNormal;
           body = bodies[i];
       }
   }
   return output;
}

//---------------------hasIntersection---------------------
// Loop through all the bodies and check if there exists
// one such that the distance between the intersection point
// and the ray origin is less than dist
//---------------------------------------------------------
bool BasicCollection::hasIntersection(Ray& ray, double distSq) const
{
    for (int i=0; i<bodies.size(); ++i)
    {
        Vector3d bodyNormal = Vector3d::Zero();
        Vector3d bodyPoint = Vector3d::Zero();
        int bodyOutput = bodies[i]->getGeometry()->intersectRay(ray, bodyPoint, bodyNormal);

        if (bodyOutput == 0) continue; //if we dont have a hit then go to the next body

        double hitDistSq = (bodyPoint - ray.getOrigin()).squaredNorm();

        if (hitDistSq < distSq) return true;
    }

    return false;
}

BasicCollection::~BasicCollection()
{
    for (int i=0; i<bodies.size(); ++i)
    {
        delete bodies[i];
    }
}