#include "Sphere.h"

//----------------------intersectRay-----------------------
//Ray-Shpere intersection.  The code is based on page 178 of
//Ericson's RTCD.  The general idea is to represent the ray
//as $p+td$ and solve for $t$ using a quadratic equation determined
//from the constraint $(p+td) \dot (p+td) = r^2$
//---------------------------------------------------------
int Sphere::intersectRay(Ray& ray, Vector3d& interPoint, Vector3d& interNormal)
{
    Vector3d m = ray.getOrigin() - position;
    double b = m.dot(ray.getDirection());
    double c = m.squaredNorm() - radius*radius;

    //if the origin is outside the sphere and pointing away from it then stop
    if (c > 0.0 && b > 0.0) return 0;

    //the discrimanent of the quadratic equation.
    double discr = b*b - c;
    //if discr is smaller than 0 then there's no solution
    if (discr < 0.0) return 0;
    //now we know there's an intersection so we want the smaller value of $t$
    //as long as it isn't negative
    discr = sqrt(discr);
    double t = -b - discr;
    if (t < 0)
    {
        t = -b + discr;
    }
    //now we need to calculate the intersection point and the normal
    interPoint = ray.getOrigin() + t*ray.getDirection();
    interNormal = interPoint - position;
    interNormal.normalize();

    //if the origin is outside the sphere
    if (c > 0.0) return 1;
    //if it's inside then return -1
    return -1;
}