#include "ImagePPM.h"
#include <iostream>
//Default ctor
ImagePPM::ImagePPM()
{
    width = 0;
    height = 0;
}

ImagePPM::ImagePPM(int width, int height)
{
    this->width = width;
    this->height = height;
    //fill up the image with zeros
    this->image = Array2D(height, vector<Color3i>(width, Color3i::Zero()));
}

void ImagePPM::readPPM(string fileName)
{
    ifstream in;
    in.open(fileName.c_str());
    if (!in.is_open())
    {
        cerr << "Couldn't open file:" << fileName << "\n";
        exit(-1);
    }
    int max;
    char type[10];
    in >> type;
    in >> width >> height;
    in >> max;

    //Start reading line by line
    image = Array2D(height, vector<Color3i>(width, Color3i::Zero()));
    for (int row = 0; row < height; row++)
        for (int col = 0; col < width; col++)
        {
            int red, green, blue;
            in >> red >> green >> blue;
            Color3i pixel(red, green, blue);
            image[row][col] = pixel;
        }
}

void ImagePPM::writePPM(ostream& out)
{
    out << "P3\n";
    out << width << " " << height << "\n";
    out << 255 << "\n"; //the maximun rgb value
    for (int row = 0; row < height; row++)
    {
        for (int col = 0; col < width; col++)
        {
            Color3i pixel = image[row][col];
            out << pixel[0] << " " << pixel[1] << " " << pixel[2] << " ";
        }
        out << "\n";
    }
}

void ImagePPM::setPixel(int row, int col, Color3i pixel)
{
    if ((row < width) && (col < height))
    {
        image[row][col] = pixel;
    } else {
        cerr << "setPixel: out of bounds";
    }
}

Color3i ImagePPM::getPixel(int row, int col)
{
    if ((row < width) && (col < height))
    {
        return image[row][col];
    } else {
        cerr << "getPixel: out of bounds";
        return Color3i::Zero();
    }
}
