/* 
 * File:   Material.h
 * Author: daniel
 *
 * Created on August 20, 2010, 6:30 PM
 */

#ifndef _MATERIAL_H
#define	_MATERIAL_H

#include "Common.h"
//-------------------------Material------------------------
//This class stores the attributes that correspond to the
//body's material
//---------------------------------------------------------

class Material {
public:
    Material() : color(Color3d::Zero()), diffuse(0), reflection(0), refraction(0),
                    refractIndex(1.0), shininess(0) {}
    Material(Color3d color, double diffuse, double reflection, double refraction,
                    double refractIndex, double shininess) :
        color(color), diffuse(clipToInterval(diffuse, 0, 1)),
        reflection(clipToInterval(reflection, 0, 1)), refraction(clipToInterval(refraction, 0, 1)),
                    refractIndex(refractIndex), shininess(shininess) {}

    Color3d getColor() const {return color;}
    double getDiffuse() const {return diffuse;}
    double getReflection() const {return reflection;}
    double getRefraction() const {return refraction;}
    double getRefractIndex() const {return refractIndex;}
    double getSpecular() const {return 1 - diffuse;}
    double getShininess() const {return shininess;}

    void setColor(Color3d color){this->color = color;}
    void setDiffuse(double diffuse){this->diffuse = clipToInterval(diffuse, 0, 1);}
    void setReflection(double reflection){this->reflection = clipToInterval(reflection, 0, 1);}
    void setRefraction(double refraction){this->refraction = clipToInterval(refraction, 0, 1);}
    void setRefractIndex(double refractIndex){this->refractIndex = refractIndex;}
    void setShininess(double shininess){this->shininess = shininess;}

private:
    Color3d color;
    double diffuse; //the percentage of diffuse light emmited, must be between 0 and 1
    double reflection; //percentage to light reflected, must be between 0 and 1
    double refraction; //also between 0 and 1
    double refractIndex; //the index of refraction
    double shininess; //the smoother the body the higher this should be
};

#endif	/* _MATERIAL_H */

