/* 
 * File:   Scene.h
 * Author: daniel
 *
 * Created on August 22, 2010, 5:33 PM
 */

#ifndef _SCENE_H
#define	_SCENE_H

#include <vector>
#include <iostream>
#include "Geometry.h"
#include "Geometry/Sphere.h"
#include "Geometry/Plane.h"
#include "BodyCollection.h"
#include "Common.h"
#include "Ray.h"
#include "Light.h"
#include "ImagePPM.h"
#include "Shader.h"

extern "C" { //include header files for lua
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
}

using namespace std;

//--------------------------Scene--------------------------
// This class holds the different parts of the scene and uses
// a shader to generate the picture.
//---------------------------------------------------------
class Shader;

class Scene {
public:
    Scene(BodyCollection* bC, Shader* shader);
    ~Scene();

    void addLight(Light* light){lights.push_back(light);}
    BodyCollection* getBodyCollection() const {return bodyCollection;}
    vector<Light*> getLights() const {return lights;}
    void generateImage() const;

    //load bodies and lights fron config file. return 0 on failure and 1 on success
    int loadConfigFile(const char* configFile);
    
private:
    BodyCollection* bodyCollection;
    Shader* shader;
    vector<Light*> lights;

    lua_State* L; //a pointer to a lua machine

    void loadLights(); //load all of the lights
    void loadBodies(); //load all of the bodies
    Body* loadBody(); //load a body from a config file
    Geometry* loadGeometry(); //load a shape from a config file
    Light* loadLight(); //load a light from the config file
    Color3d loadColor(const char* colorName); //load a color
    Vector3d loadVector(const char* vectorName); //load a vector
    int loadDouble(const char* doubleName, double& d); //load a double

    Color3i reinhardMap(Color3d& color) const;
    Color3i truncateToning(Color3d& color) const;
};
#endif	/* _SCENE_H */

