/* 
 * File:   BasicShader.h
 * Author: daniel
 *
 * Created on August 22, 2010, 5:16 PM
 */

#ifndef _BASICSHADER_H
#define	_BASICSHADER_H

#include "../Shader.h"

#define MAX_RECURSE 3 //the most levels of reflection we investigate
#define REFLECT_JUMP 0.001 //the amount we get a head start in the direction of the reflected ray

//-----------------------BasicShader-----------------------
// This implements a Shader which uses fairly primitive shading
// methods.
//---------------------------------------------------------
class BasicShader : public Shader{
public:
    //the Shader can only be initialized afer we have the bodies and lights
    BasicShader() {}

    Color3d getColor(Ray& ray, int level) const;
private:
    //calculate the diffuse lighting based on the intersection point, normal, and the body preoperties
    Color3d getPhong(Ray& ray, Vector3d& interPoint, Vector3d& normal, Body* body) const;
    //calculate the reflected light
    Color3d getReflected(Ray& ray, Vector3d& interPoint, Vector3d& normal, Body* body, int level) const;
    //calculate the refracted light
    Color3d getRefracted(Ray& ray, Vector3d& interPoint, Vector3d& normal, Body* body, int dir, int level) const;
    //determine if the ray from point in the direction lightDir intersects bodies with a squared distance of less that distSq
    //this is used to determine if the point is in a shadow of a light source located at endpoint
    bool inShadow(Vector3d& point, Vector3d& lightDir, double distSq) const;
};

#endif	/* _BASICSHADER_H */

